@@ -1,4 +1,4 @@
-package Pivot::ArrayOfHashes 1.0001;
+package Pivot::ArrayOfHashes 1.0002;
 
 # ABSTRACT: Pivot arrays of hashes, such as those returned by DBI
 
@@ -9,7 +9,8 @@ use 5.006;
 use v5.12.0;    # Before 5.006, v5.10.0 would not be understood.
 
 use UUID       qw{uuid};
-use List::Util qw{uniq};
+use List::Util qw{uniq any};
+
 use parent 'Exporter';
 our @EXPORT_OK = qw{pivot};
 
@@ -23,17 +24,25 @@ sub pivot {
     my $data_splitter = uuid();
     my $row_splitter  = uuid();
 
+    # make sure we always use the same order of keys in comparisons
+    my @key_order = sort grep {
+        my $subj = $_;
+        !any { $_ eq $subj } ( $opts{pivot_on}, $opts{pivot_into} )
+    } keys( %{ $rows->[0] } );
+
     # First, we group by the nonspecified cols.
     # We do this by creating string aggregations of the relevant data.
     my @set;
     foreach my $row (@$rows) {
         my @s;
-        foreach my $key ( sort keys(%$row) ) {
-            next if $key eq $opts{pivot_on} || $key eq $opts{pivot_into};
+        foreach my $key (@key_order) {
             push( @s,
                 ( $key // '' ) . $data_splitter . ( $row->{$key} // '' ) );
         }
         push( @set, join( $row_splitter, @s ) );
+
+        # Stash text rep of row for later
+        $row->{_hash} = join( '', map { $_ || '' } @$row{@key_order} );
     }
 
     # Next, we reverse the process into a hash after a uniq() filter.
@@ -52,22 +61,18 @@ sub pivot {
 
     # Next, we have to pivot.
     @grouped = map {
-        my $subj      = $_;
-        my @orig_keys = keys(%$subj);
+        my $subj = $_;
 
         # Make sure to null-fill all the relevant pivoted data points.
         foreach my $param (@data) {
             $subj->{$param} = undef;
         }
+        my $hash          = join( '', map { $_ || '' } @$subj{@key_order} );
+        my @relevant_rows = grep { $hash eq $_->{_hash} } @$rows;
 
-        foreach my $row (@$rows) {
+        foreach my $row (@relevant_rows) {
 
             # Append this row's info iff we are in the group.
-            next
-              unless
-              scalar( grep { ( $subj->{$_} // '' ) eq ( $row->{$_} // '' ) }
-                  @orig_keys ) == scalar(@orig_keys);
-
             my $field = $row->{ $opts{pivot_into} };
             $subj->{$field} = $row->{ $opts{pivot_on} };
         }
@@ -91,7 +96,7 @@ Pivot::ArrayOfHashes - Pivot arrays of hashes, such as those returned by DBI
 
 =head1 VERSION
 
-version 1.0001
+version 1.0002
 
 =head1 SYNOPSIS
 
